@@ -1,6 +1,9 @@
 #### "Namespace private" Auxiliaries  such as method functions
 #### (called from more than one place --> need to be defined early)
 
+.Matrix.avoiding.as.matrix <- FALSE # (always on CRAN -- have documented it since 2015)
+## NB: sync with  ../NAMESPACE
+
 ## Need to consider NAs ;  "== 0" even works for logical & complex:
 ## Note that "!x" is faster than "x == 0", but does not (yet!) work for complex
 ## if we did these in C, would gain a factor 2 (or so):
@@ -69,6 +72,9 @@ is.null.DN <- function(dn) {
 ##  not %in%  :
 `%nin%` <- function (x, table) is.na(match(x, table))
 
+nonTRUEoption <- function(ch) is.null(v <- getOption(ch)) || !isTRUE(v)
+
+
 ##' @title Check identical(i, 0:n) {or identical(i, 1:n) when Ostart is false}
 ##' @param i an integer vector, to be compared with 0:n or 1:n
 ##' @param n an integer number
@@ -148,9 +154,11 @@ MatrixClass <- function(cl, cld = getClassDef(cl),
     else { ## possibly recursively
 	r <- .selectSuperClasses(cld@contains, dropVirtual = dropVirtual,
 				 namesOnly = TRUE, ...)
-	if(length(r))
-	    Recall(r[1], ...Matrix = ...Matrix, dropVirtual = dropVirtual)
-	else r
+	if(length(r)) {
+	    while(!length(r1 <- Recall(r[1], ...Matrix = ...Matrix, dropVirtual = dropVirtual))
+		  && length(r) > 1) r <- r[-1]
+	    r1
+	} else r
     }
 }
 
@@ -514,6 +522,7 @@ indDiag <- function(n) cumsum(c(1L, rep.int(n+1L, n-1)))
 ### -----
 ### "Theory" behind this: /u/maechler/R/MM/MISC/lower-tri-w.o-matrix.R
 ## NB: also have "abIndex" version:  abIindTri() --> ./abIndex.R
+## Size problem:  indTri(n) is of size ~ n^2/2  which may be too large!
 indTri <- function(n, upper = TRUE, diag = FALSE) {
     ## Indices of (strict) upper/lower triangular part
     ## == which(upper.tri(diag(n), diag=diag) or
@@ -623,6 +632,7 @@ nnzSparse <- function(x, cl = class(x), cld = getClassDef(cl))
 ## For sparseness handling, return a
 ## 2-column (i,j) matrix of 0-based indices of non-zero entries:
 
+##' the workhorse for non0ind(.), but occasionally used directly
 non0.i <- function(M, cM = class(M), uniqT=TRUE) {
     if(extends(cM, "TsparseMatrix")) {
 	if(uniqT && is_not_uniqT(M))
@@ -635,6 +645,7 @@ non0.i <- function(M, cM = class(M), uniqT=TRUE) {
     }
 }
 
+##' the "more versatile / user" function (still not exported):
 non0ind <- function(x, cld = getClassDef(class(x)),
 		    uniqT = TRUE, xtendSymm = TRUE, check.Udiag = TRUE)
 {
@@ -971,7 +982,7 @@ l2d_meth <- function(x) {
     else .M.kindC(clx)
 }
 
-## the same as .M.kind, but also knows "i"
+##' *V*ector kind (as .M.kind, but also knows "i")
 .V.kind <- function(x, clx = class(x)) {
     ## 'clx': class() *or* class definition of x
     if(is.matrix(x) || is.atomic(x)) { ## 'old style' matrix or vector
@@ -1198,7 +1209,7 @@ mat2tri <- function(from, sparse=NA) {
     if(isTri) {
 	d <- dim(from)
 	if(is.na(sparse))
-	    sparse <- prod(d) > 2*sum(isN0(from)) ## <==> sparseDefault() in Matrix()
+	    sparse <- prod(d) > 2*sum(isN0(from)) ## <==> sparseDefault() above
 	if(sparse)
 	    as(as(from, "sparseMatrix"), "triangularMatrix")
 	else
@@ -1331,7 +1342,7 @@ isTriC <- function(object, upper = NA, ...) {
 	    ## } else { ## uplo == "L"
 	    ## }
 ### very cheap workaround
-	    all0(as.matrix(object)[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
+	    all0(as(object,"matrix")[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
         }
 }
 
@@ -1502,11 +1513,32 @@ all0Matrix <- function(n,m) {
     new(newclass, Dim = d, Dimnames = x@Dimnames, p = rep.int(0L, d[2]+1L))
 }
 
-.M.vectorSub <- function(x,i) { ## e.g. M[0] , M[TRUE],  M[1:2]
-    if(any(as.logical(i)) || prod(dim(x)) == 0)
-	## FIXME: for *large sparse*, use sparseVector !
+##' Subsetting matrix/vector in "vector style", e.g. M[0], M[TRUE], M[1:2], M[-7]
+##' @param x any matrix/Matrix/(sparse)vector, to be subset
+##' @param i integer (incl negative!) or logical 'index'.
+##' @param allowSparse logical indicating if the result may be a
+##'   \code{"sparseVector"}; the default is false for reasons of back
+##'   compatibility  (against efficiency here).
+##' @note 2018-03: Now partially based on \code{as(x, "sparseVector")[i]}
+##'   which has been improved itself.
+.M.vectorSub <- function(x, i, allowSparse=FALSE) {
+    if(prod(dim(x)) == 0)
 	as(x, "matrix")[i]
-    else ## save memory (for large sparse M):
+    else if(any(as.logical(i))) {
+	if(inherits(x, "denseMatrix"))
+	    as(x, "matrix")[i]
+	else { ## sparse ...
+	    ## if(is.logical(i)) # unfortunately, this is not-yet-implemented!
+	    ##     x[as(i, "sparseVector")]
+	    ## else if(all(i >= 0))
+	    if(is.numeric(i) && all(i >= 0))
+		subset.ij(x, ij = arrayInd(i, .dim=dim(x), useNames=FALSE))
+	    else if(allowSparse) # more efficient here
+		as(x, "sparseVector")[i]
+	    else # sparse result not allowed
+		sp2vec(as(x, "sparseVector")[i])
+	}
+    } else ## save memory (for large sparse M):
 	as.vector(x[1,1])[FALSE]
 }
 
@@ -1546,13 +1578,16 @@ setparts <- function(x,y, uniqueCheck = TRUE, check = TRUE) {
 ##' @param ...
 ##' @param which.call passed to sys.call().  A caller may use -2 if the message should
 ##' mention *its* caller
-chk.s <- function(..., which.call = -1) {
+chk.s <- function(..., which.call = -1,
+		  depCtrl = if(exists("..deparseOpts")) "niceNames")
+{
     if(nx <- length(list(...)))
 	warning(sprintf(ngettext(nx,
                                  "extra argument %s will be disregarded in\n %s",
                                  "extra arguments %s will be disregarded in\n %s"),
-                        sub(")$", '', sub("^list\\(", '', deparse(list(...), control=c()))),
-                        deparse(sys.call(which.call), control=c())),
+                        sub(")$", '', sub("^list\\(", '',
+                                          deparse(list(...), control=depCtrl))),
+                        deparse(sys.call(which.call), control=depCtrl)),
                 call. = FALSE, domain=NA)
 }
 
