@@ -39,7 +39,10 @@ Diagonal <- function(n, x = NULL)
 			    unitri, kind,
 			    cols = if(n) 0:(n - 1L) else integer(0))
 {
-    stopifnot(n == (n. <- as.integer(n)), (n <- n.) >= 0)
+    n <- if (missing(n)) length(x) else {
+	stopifnot(length(n) == 1, n == as.integer(n), n >= 0)
+	as.integer(n)
+    }
     if(!(mcols <- missing(cols)))
 	stopifnot(0 <= (cols <- as.integer(cols)), cols < n)
     m <- length(cols)
@@ -79,12 +82,13 @@ Diagonal <- function(n, x = NULL)
 }
 
 ## Pkg 'spdep' had (relatively slow) versions of this as_dsCMatrix_I()
-.symDiagonal <- function(n, x = rep.int(1,n), uplo = "U")
-    .sparseDiagonal(n, x, uplo, shape = "s")
+.symDiagonal <- function(n, x = rep.int(1,n), uplo = "U", kind)
+    .sparseDiagonal(n, x, uplo, shape = "s", kind = kind)
 
+## NOTA BENE: .triDiagonal() would be misleading (<=> banded tri-diagonal matrix !)
 # instead of   diagU2N(as(Diagonal(n), "CsparseMatrix")), diag = "N" in any case:
-.trDiagonal <- function(n, x = 1, uplo = "U", unitri=TRUE)
-    .sparseDiagonal(n, x, uplo, shape = "t", unitri=unitri)
+.trDiagonal <- function(n, x = 1, uplo = "U", unitri = TRUE, kind)
+    .sparseDiagonal(n, x, uplo, shape = "t", unitri=unitri, kind=kind)
 
 
 ## This is modified from a post of Bert Gunter to R-help on  1 Sep 2005.
@@ -351,7 +355,7 @@ setAs("matrix", "diagonalMatrix",
 	  if(d[1] != (n <- d[2])) stop("non-square matrix")
 	  if(any(from[row(from) != col(from)] != 0))
 	      stop("matrix with non-zero off-diagonals cannot be coerced to \"diagonalMatrix\"")
-	  x <- diag(from)
+	  x <- diag(from); names(x) <- NULL # don't want them in 'x' slot
 	  if(is.logical(x)) {
 	      cl <- "ldiMatrix"
 	      uni <- allTrue(x) ## uni := {is it unit-diagonal ?}
@@ -371,7 +375,7 @@ setAs("Matrix", "diagonalMatrix",
 	  if(d[1] != (n <- d[2])) stop("non-square matrix")
 	  if(!isDiagonal(from)) stop("matrix is not diagonal")
 	  ## else:
-	  x <- diag(from)
+	  x <- diag(from); names(x) <- NULL # don't want them in 'x' slot
 	  if(is.logical(x)) {
 	      cl <- "ldiMatrix"
 	      uni <- allTrue(x)
@@ -556,7 +560,7 @@ setMethod("norm", signature(x = "diagonalMatrix", type = "character"),
 	      type <- toupper(substr(type[1], 1, 1))
 	      isU <- (x@diag == "U") # unit-diagonal
 	      if(type == "F") sqrt(if(isU) n else sum(x@x^2))
-	      else { ## norm == "I","1","O","M" :
+	      else { ## norm == "I","1","O","M","2" :
 		  if(isU) 1 else max(abs(x@x))
 	      }
 	  })
@@ -1109,6 +1113,8 @@ for(arg2 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = "ddiMatrix", e2 = arg2),
 	  function(e1,e2) {
 	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+		  return(if(n) numeric() else e1)
 	      f0 <- callGeneric(0, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
@@ -1131,6 +1137,8 @@ for(arg1 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = arg1, e2 = "ddiMatrix"),
 	  function(e1,e2) {
 	      n <- e2@Dim[1L]
+	      if(length(e1) == 0L)
+		  return(if(n) numeric() else e2)
 	      f0 <- callGeneric(e1, 0)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e1)) == 1L
@@ -1154,6 +1162,9 @@ for(arg2 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = "ldiMatrix", e2 = arg2),
 	  function(e1,e2) {
 	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+		  return(if(n) numeric()
+			 else copyClass(e1, "ddiMatrix", c("diag", "Dim", "Dimnames"), check=FALSE))
 	      f0 <- callGeneric(0, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
@@ -1178,6 +1189,9 @@ for(arg1 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = arg1, e2 = "ldiMatrix"),
 	  function(e1,e2) {
 	      n <- e2@Dim[1L]
+	      if(length(e1) == 0L)
+		  return(if(n) numeric()
+			 else copyClass(e2, "ddiMatrix", c("diag", "Dim", "Dimnames"), check=FALSE))
 	      f0 <- callGeneric(e1, 0)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e1)) == 1L
@@ -1210,6 +1224,9 @@ for(arg2 in c("numeric","logical"))
 setMethod("Ops", signature(e1 = "ddiMatrix", e2 = arg2),
 	  function(e1,e2) {
 	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+		  return(if(n) logical()
+			 else copyClass(e1, "ldiMatrix", c("diag", "Dim", "Dimnames"), check=FALSE))
 	      f0 <- callGeneric(0, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
@@ -1235,6 +1252,8 @@ for(arg2 in c("numeric","logical"))
 setMethod("Ops", signature(e1 = "ldiMatrix", e2 = arg2),
 	  function(e1,e2) {
 	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+                  return(if(n) logical() else e1)
 	      f0 <- callGeneric(FALSE, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
